#!/etc/opt/openit/perl -w
#
# $Id: psm.dm,v 1.17 2001/11/01 15:49:53 borgar Exp $
#
# (C) Copyright 2006 by Open iT Norge AS, All rights reserved.
#
# This software may be used or copied only with the written permission of
# Open iT Norge AS or in accordance with the terms and conditions
# stipulated in the agreement under which this software has been supplied.
#
# This file is part of Open iT Resources.
#
#HEAD##########################################################################
#
# @NAME:
#   add_archive_type
# @AUTHOR:
#   Kjetil Yri
# @CREATED:
#   Thu Dec 14 4:48:15 MET 2006
# @DESCRIPTION:
#   Add an archive type in data_generation_config
#
#/HEAD#########################################################################

use strict;

my( $spec_file ) = @ARGV;

my $config = "/var/opt/openit/etc/data_generation_config";

print "Adding archive type to '$config'.\n";

if( -f "./files/$spec_file" ) {
    $spec_file = "./files/$spec_file";
}

my $conf_lines = &read_file( $config );
my $spec_lines = &read_file( $spec_file );

my $type = undef;
my $line;
foreach $line( @$spec_lines ) {
    if( ! defined $type && $line =~ m/^\d+:/ ) {
	die "Archive type must come first.\n";
    }
    if( $line =~ m/^TYPE:([^:]+):/ ) {
	if( defined $type ) {
	    die "Only one single archive type is allowed.\n";
	}
	$type = $1;
    }
}

foreach $line( @$conf_lines ) {
    if( $line =~ m/^TYPE:$type:/ ) {
	print "Warning: Archive type already exists. Skipping update\n";
	exit( 0 );
    }
}

push @$conf_lines, "\n";
push @$conf_lines, @$spec_lines;

open( CONF_OUT, ">$config.tmp" ) or die "Could not open '$config.tmp'.";
foreach $line( @$conf_lines ) {
    if( ! print CONF_OUT $line ) {
	my $err = $!;
	close( CONF_OUT );
	unlink "$config.tmp";
	die "Could not write to '$config.tmp'. $!";
    }
}
close( CONF_OUT );

copy_file( "$config", "$config.old", 0, 1 );
copy_file( "$config.tmp", "$config", 0, 1 );
unlink "$config.tmp";
print "'$config' updated.\n";



sub read_file {
    my( $file ) = @_;
    open( CONF_IN, "<$file" ) or die "Could not open '$file': $!\n";

    my @lines;
    @lines = <CONF_IN>;
    if( $! ) {
	my $err = $!;
	close( CONF_IN );
	die "Could not read '$config'. $!\n";
    }
    close( CONF_IN );
    return \@lines;
}

# If replace is set to 1 it expects there to be an original from which
# to read original permissions.
# If force is set then ignore errors

sub copy_file {
    my ($from, $to, $replace, $force) = @_;
    my $buf;
    my ($dev, $ino, $mode, $nlink, $uid, $gid);
    my $statfrom;
    my ($olduid, $oldgid);
    
    if (!$force && !($to =~ m/64/) && ((!$replace) && -f "$to")) {
        die "$to is already present";
    }

#   Note: We now always take uid/gid from directory and not from
#   previous file.  This is in case its a file owned by openit
#   user in a directory owned by root.
#   if ($replace) {
#       $statfrom = $to;
#   } else {
        my @p = split(/\//, $to);
        pop(@p); # Remove the filename after full path
        $statfrom = join("/", @p);
#   }
    ($dev, $ino, $mode, $nlink, $uid, $gid) = stat($statfrom);

    if (!defined($gid)) {
        die "Failed to stat $statfrom: $!";
    }

    $olduid = $>;
    $oldgid = $(;

    $) = $gid; # Change to the group of the file
    $> = $uid; # Change to the owner of the file

    if (!open(FROM, $from)) {
        die "Can't read $from: $!";
    }
    binmode(FROM);
    if (!open(TO, ">$to.$$")) {
        die "Can't write $to: $!";
    }
    binmode(TO);

    if (!$replace) {
        $mode = 0755; # A new file so we don't copy permissions of directory
    }

    while (read(FROM, $buf, 65536)) {
        print TO $buf;
    }

    close (FROM);
    close(TO);

#   Old code, we trust in umask instead
#   if (!chmod ($mode, "$to.$$")) {
#       die "Can't chmod($mode, $to.$$): $!";
#   }

    if ($replace) { # Store the original
        if (!rename($to, "$to.old")) {
            warn "Can't rename $to to $to.old: $!";
        }
    }
    if (!rename("$to.$$", $to)) {
        die "Can't rename $to.$$ to $to: $!";
    }

    # Have to set mode
    if(!chmod ($mode, $to)) {
        rename($to, "$to.failed");
        rename("$to.old", $to);
	die "Can't set mode on $to: $!, aborting patch";
    }

#   Old code, should be redundant by $> and $) but still here due to gid bug
    if (!chown ($uid, $gid, "$to")) {
        system("chgrp $gid $to");
#       warn "Can't chown($uid, $gid, $to): $!";
    }
    $> = 0; # Change uid back to root
    $) = $oldgid; # Change gid back to root

}
